@@ -4,14 +4,14 @@ use 5.008_001;
 use strict;
 use warnings;
 
-our $VERSION = '0.2008';
+our $VERSION = '0.2009';
 
-use Carp        ();
-use File::Spec  ();
-use Exporter    ();
+use Carp              ();
+use File::Spec        ();
+use Exporter          ();
+use Data::MessagePack ();
 
 use Text::Xslate::Util qw(
-    $DEBUG
     mark_raw unmark_raw
     html_escape escaped_string
     uri_escape html_builder
@@ -28,26 +28,32 @@ our @EXPORT_OK = qw(
     html_builder
 );
 
+# $format_version + $fullpath + $compiler_and_parser_options
+my $FORMAT_VERSION = '1.0';
+my $XSLATE_MAGIC   = qq{xslate;$FORMAT_VERSION;%s;%s;};
+
 # load backend (XS or PP)
 if(!exists $INC{'Text/Xslate/PP.pm'}) {
-    my $pp = ($DEBUG =~ /\b pp \b/xms or $ENV{PERL_ONLY});
+    my $pp = ($Text::Xslate::Util::DEBUG =~ /\b pp \b/xms or $ENV{PERL_ONLY});
     if(!$pp) {
         eval {
             require XSLoader;
             XSLoader::load(__PACKAGE__, $VERSION);
         };
-        die $@ if $@ && $DEBUG =~ /\b xs \b/xms; # force XS
+        die $@ if $@ && $Text::Xslate::Util::DEBUG =~ /\b xs \b/xms; # force XS
     }
     if(!__PACKAGE__->can('render')) {
         require 'Text/Xslate/PP.pm';
     }
 }
 
+# workaround warnings about numeric when it is a developpers' version
+# it must be here because the bootstrap routine requires the under bar.
+$VERSION =~ s/_//;
+
 package Text::Xslate::Engine;
 
 use Text::Xslate::Util qw(
-    $NUMBER $STRING $DEBUG
-    literal_to_value
     import_from
     make_error
 );
@@ -55,7 +61,7 @@ use Text::Xslate::Util qw(
 BEGIN {
     our @ISA = qw(Exporter);
 
-    my $dump_load = scalar($DEBUG =~ /\b dump=load \b/xms);
+    my $dump_load = scalar($Text::Xslate::Util::DEBUG =~ /\b dump=load \b/xms);
     *_DUMP_LOAD = sub(){ $dump_load };
 
     *_ST_MTIME = sub() { 9 }; # see perldoc -f stat
@@ -70,10 +76,8 @@ BEGIN {
     *_DEFAULT_CACHE_DIR = sub() { $cache_dir };
 }
 
-my $IDENT   = qr/(?: [a-zA-Z_][a-zA-Z0-9_\@]* )/xms;
+my $IDENT = qr/(?: [a-zA-Z_][a-zA-Z0-9_\@]* )/xms;
 
-# version-path-{compiler options}
-my $XSLATE_MAGIC    = qq{.xslate "%s-%s-{%s}"\n};
 
 # the real defaults are dfined in the parser
 my %parser_option = (
@@ -217,6 +221,8 @@ sub find_file {
         if(ref $p eq 'HASH') { # virtual path
             defined(my $content = $p->{$file}) or next;
             $fullpath   = \$content;
+            # TODO: we should provide a way to specify this mtime
+            #       (like as Template::Provider?)
             $orig_mtime = $^T;
             $path_id    = 'HASH';
         }
@@ -316,9 +322,8 @@ sub _load_source {
                 or Carp::carp("Xslate: Cannot make directory $cachepath (ignored): $@");
         }
 
-        # use input_layer for caches
-        if(open my($out), '>' . $self->{input_layer}, $cachepath) {
-            $self->_save_compiled($out, $asm, $fullpath);
+        if(open my($out), '>:raw', $cachepath) {
+            $self->_save_compiled($out, $asm, $fullpath, utf8::is_utf8($source));
 
             if(!close $out) {
                  Carp::carp("Xslate: Cannot close $cachepath (ignored): $!");
@@ -347,7 +352,7 @@ sub _load_compiled {
     if($self->{cache} >= 2) {
         # threshold is the most latest modified time of all the related caches,
         # so if the cache level >= 2, they seems always fresh.
-        $threshold = 9**9**9;
+        $threshold = 9**9**9; # force to purge the cache
     }
     else {
         $threshold ||= $fi->{cache_mtime};
@@ -355,63 +360,56 @@ sub _load_compiled {
     # see also tx_load_template() in xs/Text-Xslate.xs
     if(!( defined($fi->{cache_mtime}) and $self->{cache} >= 1
             and $threshold >= $fi->{orig_mtime} )) {
-        printf "  _load_compiled: no fresh cache: %s, %s", $threshold, Text::Xslate::Util::p($fi) if _DUMP_LOAD;
+        printf "  _load_compiled: no fresh cache: %s, %s",
+            $threshold, Text::Xslate::Util::p($fi) if _DUMP_LOAD;
         $fi->{cache_mtime} = undef;
         return undef;
     }
 
     my $cachepath = $fi->{cachepath};
-    open my($in), '<' . $self->{input_layer}, $cachepath
+    open my($in), '<:raw', $cachepath
         or $self->_error("LoadError: Cannot open $cachepath for reading: $!");
 
-    if(scalar(<$in>) ne $self->_magic_token($fi->{fullpath})) {
+    my $magic = $self->_magic_token($fi->{fullpath});
+    my $data;
+    read $in, $data, length($magic);
+    if($data ne $magic) {
         return undef;
     }
+    else {
+        local $/;
+        $data = <$in>;
+        close $in;
+    }
+    my $unpacker = Data::MessagePack::Unpacker->new();
+    my $offset  = $unpacker->execute($data);
+    my $is_utf8 = $unpacker->data();
+    $unpacker->reset();
+
+    $unpacker->utf8($is_utf8);
 
-    # parse assembly
     my @asm;
-    while(defined(my $s = <$in>)) {
-        next if $s =~ m{\A [ \t]* (?: \# | // )}xms; # comments
-        chomp $s;
+    while($offset < length($data)) {
+        $offset = $unpacker->execute($data, $offset);
+        my $c = $unpacker->data();
+        $unpacker->reset();
 
-        # See ::Compiler::as_assembly()
-        # "$opname $arg #$line:$file *$symbol // $comment"
-
-        my($name, $value, $line, $file, $symbol) = $s =~ m{
-            \A
-                [ \t]*
-                ($IDENT)                        # an opname
-
-                # the following components are optional
-                (?: [ \t]+ ($STRING|[+-]?$NUMBER) )? # operand
-                (?: [ \t]+ \#($NUMBER)          # line number
-                    (?: [:] ($STRING))?         # file name
-                )?
-                (?: [ \t]+ \*($STRING) )?       # symbol name
-                (?: [ \t]* // [^\n]*)?          # comments (anything)
-            \z
-        }xmsog or $self->_error("LoadError: Cannot parse assembly (line $.): $s");
-
-        $value = literal_to_value($value);
-
-        # checks the modified of dependencies
-        if($name eq 'depend') {
-            my $dep_mtime = (stat $value)[_ST_MTIME];
+        # my($name, $arg, $line, $file, $symbol) = @{$c};
+        if($c->[0] eq 'depend') {
+            my $dep_mtime = (stat $c->[1])[_ST_MTIME];
             if(!defined $dep_mtime) {
-                $dep_mtime = 9**9**9; # force reload
-                Carp::carp("Xslate: Failed to stat $value (ignored): $!");
+                Carp::carp("Xslate: Failed to stat $c->[1] (ignored): $!");
+                return undef; # purge the cache
             }
             if($dep_mtime > $threshold){
                 printf "  _load_compiled: %s(%s) is newer than %s(%s)\n",
-                    $value,     scalar localtime($dep_mtime),
+                    $c->[1],    scalar localtime($dep_mtime),
                     $cachepath, scalar localtime($threshold)
                         if _DUMP_LOAD;
-
-                return undef;
+                return undef; # purge the cache
             }
         }
-
-        push @asm, [ $name, $value, $line, $file, $symbol ];
+        push @asm, $c;
     }
 
     if(_DUMP_LOAD) {
@@ -423,29 +421,32 @@ sub _load_compiled {
 }
 
 sub _save_compiled {
-    my($self, $out, $asm, $fullpath) = @_;
-    print $out $self->_magic_token($fullpath), $self->_compiler->as_assembly($asm);
+    my($self, $out, $asm, $fullpath, $is_utf8) = @_;
+    local $\;
+    print $out $self->_magic_token($fullpath);
+    print $out Data::MessagePack->pack($is_utf8 ? 1 : 0);
+    foreach my $c(@{$asm}) {
+        print $out Data::MessagePack->pack($c);
+    }
     return;
 }
 
 sub _magic_token {
     my($self, $fullpath) = @_;
 
-    my $opt = join(',',
+    my $opt = Data::MessagePack->pack([
         ref($self->{compiler}) || $self->{compiler},
-        (map { ref $_ ? "[@{$_}]" : $_ } $self->_extract_options(\%compiler_option)),
         $self->_extract_options(\%parser_option),
-    );
+        $self->_extract_options(\%compiler_option),
+    ]);
 
     if(ref $fullpath) { # ref to content string
         require 'Digest/MD5.pm';
         my $md5 = Digest::MD5->new();
         $md5->add(${$fullpath});
-        $fullpath = ref($fullpath) . ':' . $md5->hexdigest();
+        $fullpath = join ':', ref($fullpath), $md5->hexdigest();
     }
-
-    return sprintf $XSLATE_MAGIC,
-        $VERSION, $fullpath, $opt;
+    return sprintf $XSLATE_MAGIC, $fullpath, $opt;
 }
 
 sub _extract_options {
@@ -506,7 +507,7 @@ Text::Xslate - Scalable template engine for Perl5
 
 =head1 VERSION
 
-This document describes Text::Xslate version 0.2008.
+This document describes Text::Xslate version 0.2009.
 
 =head1 SYNOPSIS
 
@@ -545,22 +546,22 @@ This document describes Text::Xslate version 0.2008.
 
 =head1 DESCRIPTION
 
-B<Text::Xslate> is a template engine tuned for persistent applications.
-This engine introduces the virtual machine paradigm. Templates are
-compiled into intermediate code, and then executed by the virtual machine.
+B<Text::Xslate> is a template engine, tuned for persistent applications,
+safe as an HTML generator, and with rich features.
 
 The concept of Xslate is strongly influenced by Text::MicroTemplate
 and Template-Toolkit 2, but the central philosophy of Xslate is different
-from them.
-That is, the philosophy is B<sandboxing> that the template logic should
-not have no access outside the template beyond your permission.
+from them. That is, the philosophy is B<sandboxing> that the template logic
+should not have no access outside the template beyond your permission.
 
 =head2 Features
 
 =head3 High performance
 
-Xslate has a virtual machine written in XS, which is highly optimized for
-rendering templates.
+This engine introduces the virtual machine paradigm. Templates are
+compiled into intermediate code, and then executed by the virtual machine,
+which is highly optimized for rendering templates. Thus, Xslate is
+much faster than any other template engines.
 
 Here is a result of F<benchmark/x-rich-env.pl> to compare various template
 engines in "rich" environment where applications are persistent and XS modules
@@ -892,7 +893,7 @@ See L<xslate(1)> for details.
 
 =head1 TEMPLATE SYNTAX
 
-Several syntaxes are provided for templates.
+There are multiple template syntaxes available in Xslate.
 
 =over
 
@@ -977,11 +978,13 @@ Xslate command:
 
 L<xlsate>
 
-The Xslate web site:
+The Xslate web site and public repository:
 
 L<http://xslate.org/>
 
-Other template modules that Xslate is influenced by:
+L<http://github.com/gfx/p5-Text-Xslate>
+
+Other template modules that Xslate has been influenced by:
 
 L<Text::MicroTemplate>
 
@@ -1008,7 +1011,8 @@ L<http://xslate.org/benchmark.html>
 =head1 ACKNOWLEDGEMENT
 
 Thanks to lestrrat for the suggestion to the interface of C<render()>,
-the contribution of App::Xslate, and a lot of suggestions.
+the contribution of Text::Xslate::Runner (was App::Xslate), and a lot of
+suggestions.
 
 Thanks to tokuhirom for the ideas, feature requests, encouragement, and bug finding.
 
@@ -1028,13 +1032,17 @@ Thanks to chiba for the bug reports and patches.
 
 Thanks to turugina for the patch to fix Win32 problems
 
+Thanks to Sam Graham to the bug reports.
+
+Thanks to Mons Anderson to the bug reports and patches.
+
 =head1 AUTHOR
 
 Fuji, Goro (gfx) E<lt>gfuji(at)cpan.orgE<gt>
 
-Makamaka Hannyaharamitu (makamaka)
+Makamaka Hannyaharamitu (makamaka) (Text::Xslate::PP)
 
-Maki, Daisuke (lestrrat)
+Maki, Daisuke (lestrrat) (Text::Xslate::Runner)
 
 =head1 LICENSE AND COPYRIGHT
 
