@@ -3,14 +3,14 @@ package Text::Xslate::PP;
 use 5.008_001;
 use strict;
 
-our $VERSION = '0.2008';
+our $VERSION = '0.2009';
+$VERSION =~ s/_//; # for developpers versions
 
 BEGIN{
     $ENV{XSLATE} = ($ENV{XSLATE} || '') . '[pp]';
 }
 use Text::Xslate::Util qw(
     $DEBUG
-    $NUMBER $STRING
     p
 );
 
@@ -629,7 +629,7 @@ Text::Xslate::PP - Yet another Text::Xslate runtime in pure Perl
 
 =head1 VERSION
 
-This document describes Text::Xslate::PP version 0.2008.
+This document describes Text::Xslate::PP version 0.2009.
 
 =head1 DESCRIPTION
 
